#include <usb.h>
#include <stdio.h>

void initialize(void) {
    usb_init();
}

int count_devices(int vendorID, int productID) {
    struct usb_bus *bus;
    struct usb_device *dev;
    int device_count;
    
    usb_find_busses();
    usb_find_devices();
    device_count = 0;

    for (bus = usb_get_busses(); bus; bus = bus->next) {
        for (dev = bus->devices; dev; dev = dev->next) {
            if ((dev->descriptor.idVendor == vendorID) && (dev->descriptor.idProduct == productID)) {
                ++device_count;
            }
        }
    }
    
    return device_count;
}

usb_dev_handle *open_device(int vendorID, int productID, int devNum) {
    struct usb_bus *bus;
    struct usb_device *dev;
    int device;
    
    usb_find_busses();
    usb_find_devices();
    device = 0;
    
    for (bus = usb_get_busses(); bus; bus = bus->next) {
        for (dev = bus->devices; dev; dev = dev->next) {
            if ((dev->descriptor.idVendor == vendorID) && (dev->descriptor.idProduct == productID)) {
                ++device;
                if (device == devNum) {
                    return usb_open(dev);
                }
            }
        }
    }
    
    fprintf(stderr, "usb.open_device could not open device %d matching vendor ID = %04x and product ID = %04X\n", devNum, vendorID, productID);
    return NULL;
}

void close_device(usb_dev_handle *dev) {
    usb_close(dev);
}

int claim_interface(usb_dev_handle *dev, int interface) {
    return usb_claim_interface(dev, interface);
}

int release_interface(usb_dev_handle *dev, int interface) {
    return usb_release_interface(dev, interface);
}

int control_write(usb_dev_handle *dev, int requestType, int request, 
                    int value, int index, int length, 
                    int byte1, int byte2, int byte3, int byte4, 
                    int byte5, int byte6, int byte7, int byte8) {
    unsigned char buffer[8];

    if (length > 8) {
        fprintf(stderr, "usb.control_write only supports data transfers of up to 8 bytes.\n");
        length = 8;
    }
    
    buffer[0] = byte1; buffer[1] = byte2; buffer[2] = byte3; buffer[3] = byte4; 
    buffer[4] = byte5; buffer[5] = byte6; buffer[6] = byte7; buffer[7] = byte8;

    return usb_control_msg(dev, requestType, request, value, index, buffer, length, 500);
}

int control_read(usb_dev_handle *dev, int requestType, int request, 
                    int value, int index, int length, 
                    int *byte1, int *byte2, int *byte3, int *byte4, 
                    int *byte5, int *byte6, int *byte7, int *byte8) {
    unsigned char buffer[8];
    int ret;

    if (length > 8) {
        fprintf(stderr, "usb.control_read only supports data transfers of up to 8 bytes.\n");
        length = 8;
    }

    ret = usb_control_msg(dev, requestType, request, value, index, buffer, length, 500);

    *byte1 = buffer[0]; *byte2 = buffer[1]; *byte3 = buffer[2]; *byte4 = buffer[3];
    *byte5 = buffer[4]; *byte6 = buffer[5]; *byte7 = buffer[6]; *byte8 = buffer[7];

    return ret;
}

int interrupt_write(usb_dev_handle *dev, int endpoint, int length, 
                    int byte1, int byte2, int byte3, int byte4,
                    int byte5, int byte6, int byte7, int byte8) {
    unsigned char buffer[8];

    if (length > 8) {
        fprintf(stderr, "usb.interrupt_write only supports data transfers of up to 8 bytes.\n");
        length = 8;
    }

    buffer[0] = byte1; buffer[1] = byte2; buffer[2] = byte3; buffer[3] = byte4; 
    buffer[4] = byte5; buffer[5] = byte6; buffer[6] = byte7; buffer[7] = byte8;

    return usb_interrupt_write(dev, endpoint, buffer, length, 500);
}

int interrupt_read(usb_dev_handle *dev, int endpoint, int length, 
                    int *byte1, int *byte2, int *byte3, int *byte4, 
                    int *byte5, int *byte6, int *byte7, int *byte8) {
    unsigned char buffer[8];
    int ret;

    if (length > 8) {
        fprintf(stderr, "usb.interrupt_read only supports data transfers of up to 8 bytes.\n");
        length = 8;
    }

    ret = usb_interrupt_read(dev, endpoint, buffer, length, 500);

    *byte1 = buffer[0]; *byte2 = buffer[1]; *byte3 = buffer[2]; *byte4 = buffer[3];
    *byte5 = buffer[4]; *byte6 = buffer[5]; *byte7 = buffer[6]; *byte8 = buffer[7];

    return ret;
}
